package jp.co.glossom.adfurikun

import io.flutter.plugin.common.MethodChannel
import jp.tjkapp.adfurikunsdk.moviereward.*
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunAppOpenAd

/**
 * アプリ起動時
 */
internal class AdfurikunAppOpenAd(var number: Int, val appId: String, val channel: MethodChannel) {

    private var appOpenAd: AdfurikunAppOpenAd? = null
    private var listener: AdfurikunAppOpenAdListener? = null

    /**
     * 初期化
     * @param appLogoIconName
     */
    fun init(appLogoIconName: String) {
        if (AdfurikunPlugin.currentActivity != null) {
            appOpenAd = AdfurikunAppOpenAd(appId, AdfurikunPlugin.currentActivity).apply {
                setAdfurikunAppOpenAdListener(createListener)
                val appLogoIconResourceId = AdfurikunCommons.getAppLogoIconResourceId(AdfurikunPlugin.currentActivity, appLogoIconName)
                if (appLogoIconResourceId > 0) {
                    initialize(appLogoIconResourceId)
                } else {
                    initialize()
                }
            }
        }
    }

    /**
     * 読み込み
     * @param timeout
     */
    fun load(timeout: Double) {
        appOpenAd?.load((timeout * 1000).toLong())
    }

    /**
     * 再生
     */
    fun play() {
        AdfurikunPlugin.currentActivity?.let {
            appOpenAd?.play(it)
        }
    }

    /**
     * 準備判定
     * @param result
     */
    fun isPrepared(result: MethodChannel.Result) {
        result.success(appOpenAd?.isPrepared ?: false)
    }

    /**
     * 破棄
     */
    fun destroy() {
        appOpenAd?.destroy()
        appOpenAd = null
        listener = null
    }

    private val createListener: AdfurikunAppOpenAdListener?
        get() {
            listener ?: run {
                listener = object : AdfurikunAppOpenAdListener {
                    override fun onPrepareSuccess(appId: String?) {
                        channel.invokeMethod(Constants.ON_PREPARE_SUCCESS, AdfurikunCommons.getArgs(number = number, appId = appId ?: ""))
                    }

                    override fun onPrepareFailure(appId: String?, error: AdfurikunMovieError?) {
                        channel.invokeMethod(Constants.ON_PREPARE_FAILURE, AdfurikunCommons.getArgs(number = number, appId = appId ?: "", errorCode = error?.errorCode ?: 0, adNetworkErrorList = error?.adNetworkErrorList))
                    }

                    override fun onStartPlaying(data: AppOpenAdData) {
                        channel.invokeMethod(Constants.ON_START_PLAYING, AdfurikunCommons.getArgs(number = number, appId = appId, adNetworkKey = data.adnetworkKey ?: ""))
                    }

                    override fun onFailedPlaying(data: AppOpenAdData, error: AdfurikunMovieError?) {
                        channel.invokeMethod(Constants.ON_FAILED_PLAYING, AdfurikunCommons.getArgs(number = number, appId = appId, adNetworkKey = data.adnetworkKey ?: "", adNetworkErrorList = error?.adNetworkErrorList))
                    }

                    override fun onAdClose(data: AppOpenAdData) {
                        channel.invokeMethod(Constants.ON_AD_CLOSE, AdfurikunCommons.getArgs(number = number, appId = appId, adNetworkKey = data.adnetworkKey ?: ""))
                    }
                }
            }
            return listener
        }
}