package jp.co.glossom.adfurikun

import android.content.Context
import jp.tjkapp.adfurikunsdk.moviereward.AdNetworkError
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunDebugUtility
import jp.tjkapp.adfurikunsdk.moviereward.AdfurikunSdk

/**
 * アドフリくん共通
 */
internal object AdfurikunCommons {

    init {
        AdfurikunSdk.init(AdfurikunPlugin.currentActivity)
    }

    /**
     * 広告音を設定
     * @param arguments
     */
    fun setSoundState(arguments: HashMap<String, *>?) {
        (arguments?.get(Constants.KEY_BOOL) as? Boolean)?.let {
            AdfurikunSdk.setAdfurikunMovieSoundState(it)
        }
    }

    /**
     * テストモードを設定
     * @param arguments
     */
    fun setTestMode(arguments: HashMap<String, *>?) {
        (arguments?.get(Constants.KEY_BOOL) as? Boolean)?.let {
            AdfurikunSdk.isAdNetworkTestMode = it
        }
    }

    /**
     * ユーザーGDPR同意を設定
     * @param arguments
     */
    fun setUserConsent(arguments: HashMap<String, *>?) {
        (arguments?.get(Constants.KEY_BOOL) as? Boolean)?.let {
            AdfurikunSdk.hasUserConsent = it
        }
    }

    /**
     * COPPAを設定
     * @param arguments
     */
    fun setChildDirected(arguments: HashMap<String, *>?) {
        (arguments?.get(Constants.KEY_BOOL) as? Boolean)?.let {
            AdfurikunSdk.isChildDirected = it
        }
    }

    /**
     * Googleファミリーポリシーを設定
     * @param arguments
     */
    fun setGoogleFamiliesPolicy(arguments: HashMap<String, *>?) {
        (arguments?.get(Constants.KEY_BOOL) as? Boolean)?.let {
            AdfurikunSdk.isGoogleFamiliesPolicy = it
        }
    }

    /**
     * プラグインバージョンを設定
     * @param arguments
     */
    fun setPluginVersion(arguments: HashMap<String, *>?) {
        val pluginVersion = (arguments?.get(Constants.KEY_PLUGIN_VERSION) as? String) ?: ""
        if (pluginVersion.isNotBlank()) {
            AdfurikunSdk.setPlatformInfo("flutter", pluginVersion)
        }
    }

    /**
     * 地域を設定
     * @param arguments
     */
    fun setRegion(arguments: HashMap<String, *>?) {
        when (((arguments?.get(Constants.KEY_STRING) as? String) ?: "")) {
            AdfurikunSdk.Region.DEVICE.name -> AdfurikunSdk.region = AdfurikunSdk.Region.DEVICE
            AdfurikunSdk.Region.JAPAN.name -> AdfurikunSdk.region = AdfurikunSdk.Region.JAPAN
            AdfurikunSdk.Region.FOREIGN.name -> AdfurikunSdk.region = AdfurikunSdk.Region.FOREIGN
        }
    }

    /**
     * 分析用データ送信を設定
     */
    fun forceSendEvent() {
        AdfurikunDebugUtility.forceSendEvent()
    }

    /**
     * 配信情報修正を設定
     * @param arguments
     */
    fun setAdNetworkInformation(arguments: HashMap<String, *>?) {
        val appId = (arguments?.get(Constants.KEY_ID) as? String) ?: ""
        if (appId.isNotBlank()) {
            (arguments?.get(Constants.KEY_AD_NETWORK_INFORMATION) as? HashMap<String, Int>)?.let {
                AdfurikunDebugUtility.setAdNetworkInformation(appId, it)
            }
        }
    }

    /**
     * Argument情報
     *
     * @param number
     * @param appId
     * @param adNetworkKey
     * @param title
     * @param description
     * @param errorCode
     */
    fun getArgs(number: Int, appId: String, adNetworkKey: String? = null, title: String? = null, description: String? = null, errorCode: Int? = null, adNetworkErrorList: List<AdNetworkError>? = null): HashMap<String, Any> {
        val args = hashMapOf<String, Any>()
        args[Constants.KEY_NUMBER] = number
        args[Constants.KEY_ID] = appId
        if (adNetworkKey != null) {
            args[Constants.KEY_AD_NETWORK_KEY] = adNetworkKey
        }
        if (title != null) {
            args[Constants.KEY_TITLE] = title
        }
        if (description != null) {
            args[Constants.KEY_DESCRIPTION] = description
        }
        if (errorCode != null) {
            args[Constants.KEY_ERROR_CODE] = errorCode
        }
        if (adNetworkErrorList != null) {
            val list = ArrayList<HashMap<String, Any>>()
            for (value in adNetworkErrorList) {
                val map = hashMapOf<String, Any>()
                map[Constants.KEY_AD_NETWORK_KEY] = value.adNetworkKey ?: ""
                map[Constants.KEY_ERROR_CODE] = value.errorCode ?: 0
                map[Constants.KEY_ERROR_MESSAGE] = value.errorMessage ?: ""
                list.add(map)
            }
            args[Constants.KEY_AD_NETWORK_ERROR_LIST] = list
        }
        return args
    }

    /**
     * dpからPixelへの変換
     *
     * @param context
     * @param dp
     * @return
     */
    fun convertDpToPixel(context: Context, dp: Int): Float {
        val metrics = context.resources.displayMetrics
        return dp * metrics.density
    }

    /**
     * ロードのタイムアウトを取得
     * @param arguments
     */
    fun getLoadWithTimeout(arguments: HashMap<String, *>?): Int {
        return (arguments?.get(Constants.KEY_TIMEOUT) as? Int) ?: 0
    }

    /**
     * ロードのタイムアウトを取得
     * @param arguments
     */
    fun getLoadWithTimeoutDouble(arguments: HashMap<String, *>?): Double {
        return (arguments?.get(Constants.KEY_TIMEOUT) as? Double) ?: 0.0
    }

    /**
     * アプリのロゴアイコンリソースIDを取得
     * @param context
     * @param name
     */
    fun getAppLogoIconResourceId(context: Context?, name: String): Int {
        var resourceId = getResourceIdentifier(context, name, "drawable")
        if (resourceId <= 0) {
            resourceId = getResourceIdentifier(context, name, "mipmap")
        }
        return resourceId
    }

    /**
     * リソースIDを取得
     * @param context
     * @param name
     * @param type
     */
    fun getResourceIdentifier(context: Context?, name: String, type: String): Int {
        context?.let {
            return it.resources.getIdentifier(name, type, it.packageName)
        }
        return -1
    }
}