import 'dart:io';

import 'package:adfurikun/appopenad.dart';
import 'package:adfurikun/ad_network_error.dart';
import 'package:adfurikun_example/constants.dart';
import 'package:flutter/material.dart';

class AppOpenAdPage extends StatefulWidget {
  AppOpenAdPage({Key? key}) : super(key: key);
  @override
  AppOpenAdPageState createState() => AppOpenAdPageState();
}

class AppOpenAdPageState extends State<AppOpenAdPage> {

  String messageLog = "";
  late AdfurikunAppOpenAd appOpenAd;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("AppOpenAd Sample"),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            FractionallySizedBox(
                widthFactor: 0.9,
                child: ElevatedButton(
                    onPressed: () => load(),
                    child: Text("Load")
                )
            ),
            FractionallySizedBox(
                widthFactor: 0.9,
                child: ElevatedButton(
                    onPressed: () => show(),
                    child: Text("Show")
                )
            ),
            FractionallySizedBox(
                widthFactor: 0.9,
                child: Text(messageLog)
            ),
          ],
        ),
      ),
    );
  }

  @override
  void initState() {
    super.initState();
    String appId = "";
    if (Platform.isAndroid) {
      appId = ANDROID_APP_OPEN_AD_APPID;
    } else {
      appId = IOS_APP_OPEN_AD_APPID;
    }
    appOpenAd = AdfurikunAppOpenAd(appId, "AppIcon");
    appOpenAd.listener = notifyCallback;
  }

  @override
  void dispose() {
    super.dispose();
    appOpenAd.destroy();
    appOpenAd.listener = null;
  }

  void load() {
    setState(() {
      messageLog = "広告の準備を開始します。" + "\n" + messageLog;
    });
    appOpenAd.load(3.0); // タイムアウト３秒
  }

  void show() async {
    bool isPrepared = await appOpenAd.isPrepared();
    if (isPrepared) {
      appOpenAd.play();
    }
  }

  void notifyCallback(AdfurikunAppOpenAdCallback? callback, String? appId, String? adNetworkKey, int? errorCode, List<AdNetworkError>? adNetworkErrorList) {
    setState(() {
      String message = "";
      if (AdfurikunAppOpenAdCallback.onPrepareSuccess == callback) {
        message = "広告の準備が完了しました。\n";
      } else if (AdfurikunAppOpenAdCallback.onPrepareFailure == callback) {
        message = "広告の準備が失敗しました。($errorCode)\n";
      } else if (AdfurikunAppOpenAdCallback.onStartPlaying == callback) {
        message = "広告の再生を開始しました。($adNetworkKey)\n";
      } else if (AdfurikunAppOpenAdCallback.onFailedPlaying == callback) {
        message = "広告の再生が中断しました。($adNetworkKey)\n";
      } else if (AdfurikunAppOpenAdCallback.onAdClose == callback) {
        message = "広告を閉じました。($adNetworkKey)\n";
      }
      messageLog = message + messageLog;
    });
  }
}