import 'dart:io';

import 'package:adfurikun/rectangle.dart';
import 'package:adfurikun/ad_network_error.dart';
import 'package:adfurikun_example/constants.dart';
import 'package:flutter/material.dart';
import 'package:fluttertoast/fluttertoast.dart';

class RectanglePage extends StatefulWidget {
  RectanglePage({Key? key}) : super(key: key);
  @override
  RectanglePageState createState() => RectanglePageState();
}

class RectanglePageState extends State<RectanglePage> {

  late AdfurikunRectangle rectangle;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Rectangle Sample"),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            FractionallySizedBox(
                widthFactor: 0.9,
                child: ElevatedButton(
                    onPressed: () => load(),
                    child: Text("Load")
                )
            ),
            FractionallySizedBox(
                widthFactor: 0.9,
                child: ElevatedButton(
                    onPressed: () => show(),
                    child: Text("Show")
                )
            ),
            Container(
                width: 300,
                height: 250,
                child: rectangle.adWidget()
            )
          ],
        ),
      ),
    );
  }

  @override
  void initState() {
    super.initState();
    String appId = "";
    if (Platform.isAndroid) {
      appId = ANDROID_RECTANGLE_APPID;
    } else {
      appId = IOS_RECTANGLE_APPID;
    }
    rectangle = AdfurikunRectangle(appId);
    rectangle.listener = notifyCallback;
  }

  @override
  void dispose() {
    super.dispose();
    rectangle.destroy();
    rectangle.listener = null;
  }

  void load() {
    rectangle.load();
  }

  void show() {
    rectangle.play();
  }

  void notifyCallback(AdfurikunRectangleCallback? callback, String? appId, String? adNetworkKey,
      String? title, String? description, int? errorCode, List<AdNetworkError>? adNetworkErrorList) {
    if (AdfurikunRectangleCallback.onRectangleLoadFinish == callback) {
      Fluttertoast.showToast(msg: "読み込み成功");
    } else if (AdfurikunRectangleCallback.onRectangleLoadError == callback) {
      Fluttertoast.showToast(msg: "読み込み失敗");
    } else if (AdfurikunRectangleCallback.onRectangleViewPlayStart == callback) {
      Fluttertoast.showToast(msg: "再生開始");
    } else if (AdfurikunRectangleCallback.onRectangleViewPlayFinish == callback) {
      Fluttertoast.showToast(msg: "再生終了");
    } else if (AdfurikunRectangleCallback.onRectangleViewPlayFail == callback) {
      Fluttertoast.showToast(msg: "再生失敗");
    } else if (AdfurikunRectangleCallback.onRectangleViewClicked == callback) {
      Fluttertoast.showToast(msg: "広告クリック");
    }
  }
}