import Foundation
import Flutter
import ADFMovieReward

// MARK: FullscreenAdDelegate Class

class FullscreenAdDelegate: NSObject, ADFmyMovieRewardDelegate {
    private let channel: FlutterMethodChannel
    private let number: Int
    
    init(_ channel: FlutterMethodChannel, _ number: Int) {
        self.channel = channel
        self.number = number
    }
    
    private func args(number: Int,
                      appId: String,
                      adNetworkKey: String? = nil,
                      errorCode: Int? = nil,
                      adNetworkErrorList: [[String: Any]]? = nil) -> Dictionary<String, Any?> {
        return [
            ADFConst.KEY_NUMBER: number,
            ADFConst.KEY_APP_ID: appId,
            ADFConst.KEY_AD_NETWORK_KEY: adNetworkKey,
            ADFConst.KEY_ERROR_CODE: errorCode,
            ADFConst.KEY_AD_NETWORK_ERROR: adNetworkErrorList
        ]
    }
    
    private func invokeMethod(_ method: String, _ args: Dictionary<String, Any?>) {
        channel.invokeMethod(method, arguments: args)
    }
    
    // MARK: ADF Reward, Interstitial Delegate Method
    
    func adsFetchCompleted(_ appID: String, isTestMode isTestMode_inApp: Bool, isManualMode: Bool) {
        print("[ADF DEBUG] call adsFetchCompleted, appId:\(appID), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID)
        invokeMethod(ADFConst.CALLBACK_ON_PREPARE_SUCCESS, args)
    }
    
    func adsFetchFailed(_ appID: String, error: Error, adnetworkError: [AdnetworkError]) {
        var errorCode: Int? = nil
        if let err = error as NSError? {
            print("[ADF DEBUG] call adsFetchFailed, appId:\(appID), number:\(number), error:\(err.localizedDescription)")
            errorCode = err.code
        }
        var adnetworkErrorList: [[String: Any]] = []
        for e in adnetworkError {
            var error: [String: Any] = [:]
            error[ADFConst.KEY_AD_NETWORK_KEY] = e.adnetworkKey
            error[ADFConst.KEY_ERROR_CODE] = e.errorCode
            error[ADFConst.KEY_ERROR_MESSAGE] = e.errorMessage
            adnetworkErrorList.append(error)
        }
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID,
                                                  errorCode: errorCode,
                                                  adNetworkErrorList: adnetworkErrorList)
        invokeMethod(ADFConst.CALLBACK_ON_PREPARE_FAILURE, args)
    }
    
    public func adsDidShow(_ appID: String, adNetworkKey: String) {
        print("[ADF DEBUG] call adsDidShow, appId:\(appID), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID,
                                                  adNetworkKey: adNetworkKey)
        invokeMethod(ADFConst.CALLBACK_ON_START_PLAYING, args)
    }
    
    public func adsDidCompleteShow(_ appID: String) {
        print("[ADF DEBUG] call adsDidCompleteShow, appId:\(appID), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID)
        invokeMethod(ADFConst.CALLBACK_ON_FINISHED_PLAYING, args)
    }
    
    public func adsDidHide(_ appID: String) {
        print("[ADF DEBUG] call adsDidHide, appId:\(appID), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID)
        invokeMethod(ADFConst.CALLBACK_ON_AD_CLOSE, args)
    }
    
    public func adsPlayFailed(_ appID: String, adnetworkError: AdnetworkError) {
        print("[ADF DEBUG] call adsPlayFailed, appId:\(appID), number:\(number)")
        var adnetworkErrorList: [[String: Any]] = []
        var error: [String: Any] = [:]
        error[ADFConst.KEY_AD_NETWORK_KEY] = adnetworkError.adnetworkKey
        error[ADFConst.KEY_ERROR_CODE] = adnetworkError.errorCode
        error[ADFConst.KEY_ERROR_MESSAGE] = adnetworkError.errorMessage
        adnetworkErrorList.append(error)
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID,
                                                  adNetworkErrorList: adnetworkErrorList)
        invokeMethod(ADFConst.CALLBACK_ON_FAILED_PLAYING, args)
    }
}

// MARK: AppOpenAdDelegate Class

class AppOpenAdDelegate: NSObject, ADFmyAppOpenAdDelegate {
    private let channel: FlutterMethodChannel
    private let number: Int
    
    init(_ channel: FlutterMethodChannel, _ number: Int) {
        self.channel = channel
        self.number = number
    }
    
    private func args(number: Int,
                      appId: String,
                      adNetworkKey: String? = nil,
                      errorCode: Int? = nil,
                      adNetworkErrorList: [[String: Any]]? = nil) -> Dictionary<String, Any?> {
        return [
            ADFConst.KEY_NUMBER: number,
            ADFConst.KEY_APP_ID: appId,
            ADFConst.KEY_AD_NETWORK_KEY: adNetworkKey,
            ADFConst.KEY_ERROR_CODE: errorCode,
            ADFConst.KEY_AD_NETWORK_ERROR: adNetworkErrorList
        ]
    }
    
    private func invokeMethod(_ method: String, _ args: Dictionary<String, Any?>) {
        channel.invokeMethod(method, arguments: args)
    }
    
    // MARK: ADF AppOpenAd Delegate Method
    
    func adsFetchCompleted(_ appID: String) {
        print("[ADF DEBUG] call adsFetchCompleted, appId:\(appID), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID)
        invokeMethod(ADFConst.CALLBACK_ON_PREPARE_SUCCESS, args)
    }
    
    func adsFetchFailed(_ appID: String, error: Error, adnetworkError: [AdnetworkError]) {
        var errorCode: Int? = nil
        if let err = error as NSError? {
            print("[ADF DEBUG] call adsFetchFailed, appId:\(appID), number:\(number), error:\(err.localizedDescription)")
            errorCode = err.code
        }
        var adnetworkErrorList: [[String: Any]] = []
        for e in adnetworkError {
            var error: [String: Any] = [:]
            error[ADFConst.KEY_AD_NETWORK_KEY] = e.adnetworkKey
            error[ADFConst.KEY_ERROR_CODE] = e.errorCode
            error[ADFConst.KEY_ERROR_MESSAGE] = e.errorMessage
            adnetworkErrorList.append(error)
        }
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID,
                                                  errorCode: errorCode,
                                                  adNetworkErrorList: adnetworkErrorList)
        invokeMethod(ADFConst.CALLBACK_ON_PREPARE_FAILURE, args)
    }
    
    func adsDidShow(_ appID: String, adNetworkKey: String) {
        print("[ADF DEBUG] call adsDidShow, appId:\(appID), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID,
                                                  adNetworkKey: adNetworkKey)
        invokeMethod(ADFConst.CALLBACK_ON_START_PLAYING, args)
    }
    
    func adsDidHide(_ appID: String) {
        print("[ADF DEBUG] call adsDidHide, appId:\(appID), number:\(number)")
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID)
        invokeMethod(ADFConst.CALLBACK_ON_AD_CLOSE, args)
    }
    
    func adsPlayFailed(_ appID: String, adnetworkError: AdnetworkError) {
        print("[ADF DEBUG] call adsPlayFailed, appId:\(appID), number:\(number)")
        var adnetworkErrorList: [[String: Any]] = []
        var error: [String: Any] = [:]
        error[ADFConst.KEY_AD_NETWORK_KEY] = adnetworkError.adnetworkKey
        error[ADFConst.KEY_ERROR_CODE] = adnetworkError.errorCode
        error[ADFConst.KEY_ERROR_MESSAGE] = adnetworkError.errorMessage
        adnetworkErrorList.append(error)
        let args: Dictionary<String, Any?> = args(number: number,
                                                  appId: appID,
                                                  adNetworkErrorList: adnetworkErrorList)
        invokeMethod(ADFConst.CALLBACK_ON_FAILED_PLAYING, args)
    }
}

// MARK: FullscreenAdManager Class

class FullscreenAdManager: NSObject {
    private let channel: FlutterMethodChannel
    
    private var rewardMap: Dictionary<Int, ADFmyMovieReward> = [:]
    private var interstitialMap: Dictionary<Int, ADFmyInterstitial> = [:]
    private var appopenadMap: Dictionary<Int, ADFmyAppOpenAd> = [:]
    
    private var delegateMap: Dictionary<Int, FullscreenAdDelegate> = [:]
    private var openadDelegateMap: Dictionary<Int, AppOpenAdDelegate> = [:]
    
    init(_ channel: FlutterMethodChannel) {
        self.channel = channel
    }
    
    func checkMethod(_ call: FlutterMethodCall, _ checkString: String) -> Bool {
        return call.method.hasPrefix(checkString + ADFConst.SEPARATOR_METHOD)
    }
    
    func handle(_ call: FlutterMethodCall, _ result: @escaping FlutterResult, _ adType: String) {
        guard let args = call.arguments as? [String: Any?],
              let number = args[ADFConst.KEY_NUMBER] as? Int,
              let appId = args[ADFConst.KEY_APP_ID] as? String else {
            print("[ADF WARNING] FullscreenAdManager:handle args is abnormal")
            return
        }
        if adType == ADFConst.AD_TYPE_REWARD {
            if checkMethod(call, ADFConst.METHOD_INIT) {
                let pluginVersion = args[ADFConst.KEY_PLUGIN_VERSION] as? String
                _ = createRewardIfNeeded(number, appId, pluginVersion)
            } else if checkMethod(call, ADFConst.METHOD_LOAD) {
                let reward = getReward(number)
                if let timeout = args[ADFConst.KEY_TIMEOUT] as? Int32, timeout > 0 {
                    reward?.load(withTimeout: timeout)
                } else {
                    reward?.load()
                }
            } else if checkMethod(call, ADFConst.METHOD_IS_PREPARED) {
                if let reward = getReward(number) {
                    result(reward.isPrepared())
                } else {
                    result(false)
                }
            } else if checkMethod(call, ADFConst.METHOD_PLAY) {
                let reward = getReward(number)
                if let customParams = args[ADFConst.KEY_CUSTOM_PARAMS] as? Dictionary<String, String> {
                    reward?.play(withCustomParam: customParams)
                } else {
                    reward?.play()
                }
            } else if checkMethod(call, ADFConst.METHOD_DESTROY) {
                getReward(number)?.dispose()
                removeReward(number)
            }
        } else if adType == ADFConst.AD_TYPE_INTERSTITIAL {
            if checkMethod(call, ADFConst.METHOD_INIT) {
                let pluginVersion = args[ADFConst.KEY_PLUGIN_VERSION] as? String
                _ = createInterstitialIfNeeded(number, appId, pluginVersion)
            } else if checkMethod(call, ADFConst.METHOD_LOAD) {
                let interstitial = getInterstitial(number)
                if let timeout = args[ADFConst.KEY_TIMEOUT] as? Int32, timeout > 0 {
                    interstitial?.load(withTimeout: timeout)
                } else {
                    interstitial?.load()
                }
            } else if checkMethod(call, ADFConst.METHOD_IS_PREPARED) {
                if let interstitial = getInterstitial(number) {
                    result(interstitial.isPrepared())
                } else {
                    result(false)
                }
            } else if checkMethod(call, ADFConst.METHOD_PLAY) {
                let interstitial = getInterstitial(number)
                if let customParams = args[ADFConst.KEY_CUSTOM_PARAMS] as? Dictionary<String, String> {
                    interstitial?.play(withCustomParam: customParams)
                } else {
                    interstitial?.play()
                }
            } else if checkMethod(call, ADFConst.METHOD_DESTROY) {
                getInterstitial(number)?.dispose()
                removeInterstitial(number)
            }
        } else if adType == ADFConst.AD_TYPE_APPOPENAD {
            if checkMethod(call, ADFConst.METHOD_INIT) {
                let pluginVersion = args[ADFConst.KEY_PLUGIN_VERSION] as? String
                let appLogoImageName = args[ADFConst.KEY_APP_LOGO_ICON_NAME] as? String
                _ = createAppOpenAdIfNeeded(number, appId, pluginVersion, appLogoImageName)
            } else if checkMethod(call, ADFConst.METHOD_LOAD) {
                let appopenad = getAppOpenAd(number)
                if let timeout = args[ADFConst.KEY_TIMEOUT] as? Float, timeout > 0 {
                    appopenad?.load(withTimeout: timeout)
                } else {
                    print("timeout is nothing")
                }
            } else if checkMethod(call, ADFConst.METHOD_IS_PREPARED) {
                if let appopenad = getAppOpenAd(number) {
                    result(appopenad.isPrepared())
                } else {
                    result(false)
                }
            } else if checkMethod(call, ADFConst.METHOD_PLAY) {
                let rootViewController = UIApplication.shared.windows.filter({ (w) -> Bool in
                            return w.isHidden == false
                 }).first?.rootViewController
                if let vc = rootViewController {
                    let appopenad = getAppOpenAd(number)
                    appopenad?.play(withPresenting: vc, window: nil)
                }
            } else if checkMethod(call, ADFConst.METHOD_DESTROY) {
                getAppOpenAd(number)?.dispose()
                removeAppOpenAd(number)
            }
        }
    }
    
    // MARK: Private Method
    
    private func createRewardIfNeeded(_ number: Int, _ appId: String, _ pluginVersion: String?) -> ADFmyMovieReward? {
        if let reward = rewardMap[number] {
            return reward
        }
        let delegate = FullscreenAdDelegate(channel, number)
        delegateMap.updateValue(delegate, forKey: number)
        let option: [AnyHashable: Any] = [ADFConst.KEY_PLUGIN:
                                            [ADFConst.KEY_TYPE: ADFConst.VALUE_FLUTTER,
                                             ADFConst.KEY_VERSION: pluginVersion ?? ""]]
        ADFmyMovieReward.initialize(withAppID: appId, option: option)
        if let reward = ADFmyMovieReward.getInstance(appId, delegate: delegate) {
            rewardMap.updateValue(reward, forKey: number)
            return reward
        }
        return nil
    }
    
    private func getReward(_ number: Int) -> ADFmyMovieReward? {
        return rewardMap[number]
    }
    
    private func removeReward(_ number: Int) {
        rewardMap.removeValue(forKey: number)
        delegateMap.removeValue(forKey: number)
    }
    
    private func createInterstitialIfNeeded(_ number: Int, _ appId: String, _ pluginVersion: String?) -> ADFmyInterstitial? {
        if let interstitial = interstitialMap[number] {
            return interstitial
        }
        let delegate = FullscreenAdDelegate(channel, number)
        delegateMap.updateValue(delegate, forKey: number)
        ADFmyInterstitial.initialize(withAppID: appId, option: [ADFConst.KEY_PLUGIN: pluginVersion ?? ""])
        if let interstitial = ADFmyInterstitial.getInstance(appId, delegate: delegate) {
            interstitialMap.updateValue(interstitial, forKey: number)
            return interstitial
        }
        return nil
    }
    
    private func getInterstitial(_ number: Int) -> ADFmyInterstitial? {
        return interstitialMap[number]
    }
    
    private func removeInterstitial(_ number: Int) {
        interstitialMap.removeValue(forKey: number)
        delegateMap.removeValue(forKey: number)
    }
    
    private func createAppOpenAdIfNeeded(_ number: Int, _ appId: String, _ pluginVersion: String?, _ appLogoIconName: String?) -> ADFmyAppOpenAd {
        if let appopenad = appopenadMap[number] {
            return appopenad
        }
        let delegate = AppOpenAdDelegate(channel, number)
        openadDelegateMap.updateValue(delegate, forKey: number)
        var appLogoImage: UIImage?
        if let appLogoIconName = appLogoIconName {
            appLogoImage = UIImage(named:appLogoIconName)
        }
        ADFmyAppOpenAd.initialize(withAppID: appId, appLogoImage: appLogoImage, option: [ADFConst.KEY_PLUGIN: pluginVersion ?? ""])
        let appopenad: ADFmyAppOpenAd = ADFmyAppOpenAd.getInstance(appId, delegate: delegate)
        appopenadMap.updateValue(appopenad, forKey: number)
        return appopenad
    }
    
    private func getAppOpenAd(_ number: Int) -> ADFmyAppOpenAd? {
        return appopenadMap[number]
    }
    
    private func removeAppOpenAd(_ number: Int) {
        appopenadMap.removeValue(forKey: number)
        openadDelegateMap.removeValue(forKey: number)
    }
}
