import Foundation
import Flutter
import ADFMovieReward

class OptionsManager {
    static func handle(_ call: FlutterMethodCall, _ result: @escaping FlutterResult) {
        switch call.method {
        case ADFConst.METHOD_SET_TEST_MODE:
            if let args = call.arguments as? [String: Any?],
               let mode = args[ADFConst.KEY_BOOL] as? Bool {
                ADFMovieOptions.setTestMode(mode)
            }
            break
        case ADFConst.METHOD_SET_USER_CONSENT:
            if let args = call.arguments as? [String: Any?],
               let hasConsent = args[ADFConst.KEY_BOOL] as? Bool {
                ADFMovieOptions.setHasUserConsent(hasConsent)
            }
            break
        case ADFConst.METHOD_SET_CHILD_DIRECTED:
            if let args = call.arguments as? [String: Any?],
               let childDirected = args[ADFConst.KEY_BOOL] as? Bool {
                ADFMovieOptions.isChildDirected(childDirected)
            }
            break
        case ADFConst.METHOD_SET_SOUND_STATE:
            if let args = call.arguments as? [String: Any?],
               let state = args[ADFConst.KEY_BOOL] as? Bool {
                if state == true {
                    ADFMovieOptions.setSoundState(.on)
                } else {
                    ADFMovieOptions.setSoundState(.off)
                }
            } else {
                ADFMovieOptions.setSoundState(.default)
            }
            break
        case ADFConst.METHOD_SET_REGION:
            if let args = call.arguments as? [String: Any?],
               let region = args[ADFConst.KEY_STRING] as? String {
                if region == ADFConst.ENUM_REGION_JAPAN {
                    ADFMovieOptions.setRegion(.japan)
                } else if region == ADFConst.ENUM_REGION_FOREIGN {
                    ADFMovieOptions.setRegion(.foreign)
                } else if region == ADFConst.ENUM_REGION_DEVICE {
                    ADFMovieOptions.setRegion(.deviceSetting)
                }
            }
            break
        case ADFConst.METHOD_OUTPUT_DETAILED_LOG:
            ADFLogger.logLevel = .verbose
            break
        case ADFConst.METHOD_FORCE_SENT_EVENT:
            ADFDebugUtility.setForceSendEvent()
            print("[ADF WARNING] Please remove forceSendEvent call in production")
            break
        case ADFConst.METHOD_SET_ADNETWORK_INFORMATION:
            if let args = call.arguments as? [String: Any?],
               let appId = args[ADFConst.KEY_APP_ID] as? String,
               let dic = args[ADFConst.KEY_AD_NETWORK_INFORMATION] as? [String: NSNumber] {
                ADFDebugUtility.setAdnetworkInformation(appId, info: dic)
                print("[ADF WARNING] Please remove setAdNetworkInformation call in production")
            }
            break
        default:
            print("[ADF WARNING] \(call.method) method is not implemented")
        }
    }
}
