import 'dart:io';
import 'package:flutter/services.dart';
import 'constants.dart';
import 'ad_base.dart';
import 'adfurikun_plugin.dart';
import 'ad_network_error.dart';

enum AdfurikunAppOpenAdCallback {
  onPrepareSuccess,
  onPrepareFailure,
  onStartPlaying,
  onAdClose,
  onFailedPlaying
}

typedef AdfurikunAppOpenAdListener(
    AdfurikunAppOpenAdCallback? callback,
    String? appId,
    String? adNetworkKey,
    int? errorCode,
    List<AdNetworkError>? adNetworkErrorList);

class AdfurikunAppOpenAd extends AdfurikunAdBase {
  AdfurikunAppOpenAdListener? listener;

  Map<String, AdfurikunAppOpenAdCallback> _callbackMap = {
    CALLBACK_ON_PREPARE_SUCCESS: AdfurikunAppOpenAdCallback.onPrepareSuccess,
    CALLBACK_ON_PREPARE_FAILURE: AdfurikunAppOpenAdCallback.onPrepareFailure,
    CALLBACK_ON_START_PLAYING: AdfurikunAppOpenAdCallback.onStartPlaying,
    CALLBACK_ON_AD_CLOSE: AdfurikunAppOpenAdCallback.onAdClose,
    CALLBACK_ON_FAILED_PLAYING: AdfurikunAppOpenAdCallback.onFailedPlaying
  };

  AdfurikunAppOpenAd(String appId, [String? appLogoIconName]) : super(appId, AD_TYPE_APP_OPEN_AD) {
    if (appLogoIconName?.isNotEmpty == true) {
      AdfurikunPlugin.invoke(method(METHOD_INIT), argsWithPluginVersionAndAppLogoIconName(appLogoIconName!));
    } else {
      AdfurikunPlugin.invoke(method(METHOD_INIT), argsWithPluginVersion());
    }
  }

  void load(double timeout) {
    Map<String, Object> map = args();
    map[KEY_TIMEOUT] = timeout;
    AdfurikunPlugin.invoke(method(METHOD_LOAD), map);
  }

  void play() {
    AdfurikunPlugin.invoke(method(METHOD_PLAY), args());
  }

  Future<bool> isPrepared() async {
    return await AdfurikunPlugin.invokeReturnBool(method(METHOD_IS_PREPARED), args());
  }

  @override
  void destroy() {
    super.destroy();
    AdfurikunPlugin.invoke(method(METHOD_DESTROY), args());
  }

  @override
  void notifyListener(MethodCall call) {
    if (listener != null) {
      AdfurikunAppOpenAdCallback? callback = _callbackMap[call.method];
      String? appId = call.arguments[KEY_APP_ID];
      String? adNetworkKey = call.arguments[KEY_AD_NETWORK_KEY];
      int? errorCode = call.arguments[KEY_ERROR_CODE];
      List<AdNetworkError>? adNetworkErrorList = convertAdNetworkErrorList(call.arguments[KEY_AD_NETWORK_ERROR_LIST]);
      listener!(callback, appId, adNetworkKey, errorCode, adNetworkErrorList);
    } else {
      print('AdfurikunAppOpenAdListener is null');
    }
  }
}
