import 'dart:io';
import 'package:flutter/services.dart';
import 'constants.dart';
import 'ad_base.dart';
import 'adfurikun_plugin.dart';
import 'ad_network_error.dart';

enum AdfurikunRewardCallback {
  onPrepareSuccess,
  onPrepareFailure,
  onStartPlaying,
  onFinishedPlaying,
  onAdClose,
  onFailedPlaying
}

typedef AdfurikunRewardListener(
    AdfurikunRewardCallback? callback,
    String? appId,
    String? adNetworkKey,
    int? errorCode,
    List<AdNetworkError>? adNetworkErrorList);

class AdfurikunReward extends AdfurikunAdBase {
  AdfurikunRewardListener? listener;

  Map<String, AdfurikunRewardCallback> _callbackMap = {
    CALLBACK_ON_PREPARE_SUCCESS: AdfurikunRewardCallback.onPrepareSuccess,
    CALLBACK_ON_PREPARE_FAILURE: AdfurikunRewardCallback.onPrepareFailure,
    CALLBACK_ON_START_PLAYING: AdfurikunRewardCallback.onStartPlaying,
    CALLBACK_ON_FINISHED_PLAYING: AdfurikunRewardCallback.onFinishedPlaying,
    CALLBACK_ON_AD_CLOSE: AdfurikunRewardCallback.onAdClose,
    CALLBACK_ON_FAILED_PLAYING: AdfurikunRewardCallback.onFailedPlaying
  };

  AdfurikunReward(String appId) : super(appId, AD_TYPE_REWARD) {
    AdfurikunPlugin.invoke(method(METHOD_INIT), argsWithPluginVersion());
  }

  void load() {
    AdfurikunPlugin.invoke(method(METHOD_LOAD), args());
  }

  void loadWithTimeout(int timeout) {
    Map<String, Object> map = args();
    map[KEY_TIMEOUT] = timeout;
    AdfurikunPlugin.invoke(method(METHOD_LOAD), map);
  }

  void play() {
    AdfurikunPlugin.invoke(method(METHOD_PLAY), args());
  }

  void playWithCustomParams(Map<String, String> customParams) {
    Map<String, Object> map = args();
    map[KEY_CUSTOM_PARAMS] = customParams;
    AdfurikunPlugin.invoke(method(METHOD_PLAY), map);
  }

  Future<bool> isPrepared() async {
    return await AdfurikunPlugin.invokeReturnBool(method(METHOD_IS_PREPARED), args());
  }

  @override
  void destroy() {
    super.destroy();
    AdfurikunPlugin.invoke(method(METHOD_DESTROY), args());
  }

  @override
  void notifyListener(MethodCall call) {
    if (listener != null) {
      AdfurikunRewardCallback? callback = _callbackMap[call.method];
      String? appId = call.arguments[KEY_APP_ID];
      String? adNetworkKey = call.arguments[KEY_AD_NETWORK_KEY];
      int? errorCode = call.arguments[KEY_ERROR_CODE];
      List<AdNetworkError>? adNetworkErrorList = convertAdNetworkErrorList(call.arguments[KEY_AD_NETWORK_ERROR_LIST]);
      listener!(callback, appId, adNetworkKey, errorCode, adNetworkErrorList);
    } else {
      print('AdfurikunRewardListener is null');
    }
  }
}
